import 'dart:convert';

class Address {
  String? addressType; // Cusomter; supplier; ...
  String? street;
  String? houseNumber;
  String? zip;
  String? city;
  String? state;
  String? country;

  Address({
    this.addressType,
    this.street,
    this.houseNumber,
    this.zip,
    this.city,
    this.state,
    this.country,
  });

  String getStreetAndNumber() {
    return _joinWithSeparator(street, houseNumber);
  }

  String getZipAndCity() {
    return _joinWithSeparator(zip, city);
  }

  String _joinWithSeparator(String? first, String? second, [String separator = ' ']) {
    first ??= '';
    second = second == null ? '' : '$separator$second';
    return '$first$second';
  }

  bool get hasAddress {
    return street != null || houseNumber != null || zip != null || city != null || state != null || country != null;
  }

  String asSingleLineString() {
    return _joinWithSeparator(getStreetAndNumber(), getZipAndCity(), ',');
  }

  Address copyWith({
    String? addressType,
    String? street,
    String? houseNumber,
    String? zip,
    String? city,
    String? state,
    String? country,
  }) {
    return Address(
      addressType: addressType ?? this.addressType,
      street: street ?? this.street,
      houseNumber: houseNumber ?? this.houseNumber,
      zip: zip ?? this.zip,
      city: city ?? this.city,
      state: state ?? this.state,
      country: country ?? this.country,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'addressType': addressType,
      'street': street,
      'houseNumber': houseNumber,
      'zip': zip,
      'city': city,
      'state': state,
      'country': country,
    };
  }

  factory Address.fromMap(Map<String, dynamic> map) {
    return Address(
      addressType: map['addressType'],
      street: map['street'],
      houseNumber: map['houseNumber'],
      zip: map['zip'],
      city: map['city'],
      state: map['state'],
      country: map['country'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Address.fromJson(String source) => Address.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Address(addressType: $addressType, street: $street, houseNumber: $houseNumber, zip: $zip, city: $city, state: $state, country: $country)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
  
    return other is Address &&
      other.addressType == addressType &&
      other.street == street &&
      other.houseNumber == houseNumber &&
      other.zip == zip &&
      other.city == city &&
      other.state == state &&
      other.country == country;
  }

  @override
  int get hashCode {
    return addressType.hashCode ^
      street.hashCode ^
      houseNumber.hashCode ^
      zip.hashCode ^
      city.hashCode ^
      state.hashCode ^
      country.hashCode;
  }
}
