import 'dart:convert';

import 'Address.dart';
import 'GeoCoordinates.dart';

class AddressWithGeoLocation {
  final Address address;
  GeoCoordinates? location;
  AddressWithGeoLocation({
    required this.address,
    this.location,
  });

  factory AddressWithGeoLocation.fromJson(String source) => AddressWithGeoLocation.fromMap(json.decode(source));

  factory AddressWithGeoLocation.fromMap(Map<String, dynamic> map) {
    return AddressWithGeoLocation(
      address: Address.fromMap(map['address']),
      location: GeoCoordinates.fromMap(map['location']),
    );
  }

  @override
  int get hashCode => address.hashCode ^ location.hashCode;

  bool get hasValues {
    return address.hasAddress == true || location != null;
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
  
    return other is AddressWithGeoLocation &&
      other.address == address &&
      other.location == location;
  }

  AddressWithGeoLocation copyWith({
    Address? address,
    GeoCoordinates? location,
  }) {
    return AddressWithGeoLocation(
      address: address ?? this.address,
      location: location ?? this.location,
    );
  }

  String toJson() => json.encode(toMap());

  Map<String, dynamic> toMap() {
    return {
      'address': address.toMap(),
      'location': location?.toMap(),
    };
  }

  @override
  String toString() => 'AddressWithGeoLocation(address: $address, location: $location)';
}
